@@ -29,7 +29,7 @@ import shutil
 from hashlib import md5
 import luigi
 import luigi.hdfs
-import interface
+import configuration
 import warnings
 import mrrunner
 
@@ -150,31 +150,54 @@ class HadoopJobError(RuntimeError):
         self.err = err
 
 
-def run_and_track_hadoop_job(arglist):
+def run_and_track_hadoop_job(arglist, tracking_url_callback=None, env=None):
     ''' Runs the job by invoking the command from the given arglist. Finds tracking urls from the output and attempts to fetch
     errors using those urls if the job fails. Throws HadoopJobError with information about the error (including stdout and stderr
     from the process) on failure and returns normally otherwise.
     '''
     logger.info(' '.join(arglist))
 
-    def track_process(arglist):
-        proc = subprocess.Popen(arglist, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+    def track_process(arglist, tracking_url_callback, env=None):
+        # Dump stdout to a temp file, poll stderr and log it
+        temp_stdout = tempfile.TemporaryFile()
+        proc = subprocess.Popen(arglist, stdout=temp_stdout, stderr=subprocess.PIPE, env=env)
 
         # We parse the output to try to find the tracking URL.
         # This URL is useful for fetching the logs of the job.
         tracking_url = None
+        job_id = None
         err_lines = []
-        while proc.poll() is None:
-            err_line = proc.stderr.readline()
+        while True:
+            try:
+                if proc.poll() is not None:
+                    break
+                err_line = proc.stderr.readline()
+            except KeyboardInterrupt:
+                if job_id:
+                    logger.info('Job interrupted by user, killing job %s', job_id)
+                    kill_job(job_id)
+                raise
             err_lines.append(err_line)
             if err_line.strip():
                 logger.info(err_line.strip())
             if err_line.find('Tracking URL') != -1:
                 tracking_url = err_line.strip().split('Tracking URL: ')[-1]
+                try:
+                    tracking_url_callback(tracking_url)
+                except Exception as e:
+                    logger.error("Error in tracking_url_callback, disabling! %s" % e)
+                    tracking_url_callback = lambda x: None
+            if err_line.find('Running job') != -1:
+                # hadoop jar output
+                job_id = err_line.strip().split('Running job: ')[-1]
+            if err_line.find('submitted hadoop job:') != -1:
+                # scalding output
+                job_id = err_line.strip().split('submitted hadoop job: ')[-1]
 
         # Read the rest + stdout
         err = ''.join(err_lines + [err_line for err_line in proc.stderr])
-        out = ''.join([out_line for out_line in proc.stdout])
+        temp_stdout.seek(0)
+        out = ''.join(temp_stdout.readlines())
 
         if proc.returncode == 0:
             return
@@ -195,7 +218,14 @@ def run_and_track_hadoop_job(arglist):
         else:
             raise HadoopJobError(message + 'Output from tasks below:\n%s' % task_failures, out, err)
 
-    track_process(arglist)
+    if tracking_url_callback is None:
+        tracking_url_callback = lambda x: None
+
+    track_process(arglist, tracking_url_callback, env)
+
+
+def kill_job(job_id):
+    subprocess.call(['mapred', 'job', '-kill', job_id])
 
 
 def fetch_task_failures(tracking_url):
@@ -266,7 +296,7 @@ class HadoopJobRunner(JobRunner):
         if runner_path.endswith("pyc"):
             runner_path = runner_path[:-3] + "py"
 
-        base_tmp_dir = interface.get_config().get('core', 'tmp-dir', '/tmp/luigi')
+        base_tmp_dir = configuration.get_config().get('core', 'tmp-dir', '/tmp/luigi')
         self.tmp_dir = os.path.join(base_tmp_dir, 'hadoop_job_%016x' % random.getrandbits(64))
         logger.debug("Tmp dir: %s", self.tmp_dir)
         os.makedirs(self.tmp_dir)
@@ -359,8 +389,7 @@ class HadoopJobRunner(JobRunner):
 class DefaultHadoopJobRunner(HadoopJobRunner):
     ''' The default job runner just reads from config and sets stuff '''
     def __init__(self):
-        import interface
-        config = interface.get_config()
+        config = configuration.get_config()
         streaming_jar = config.get('hadoop', 'streaming-jar')
         super(DefaultHadoopJobRunner, self).__init__(streaming_jar=streaming_jar)
         # TODO: add more configurable options
@@ -440,7 +469,12 @@ class BaseHadoopJobTask(luigi.Task):
         jcs.append('mapred.reduce.tasks=%s' % self.n_reduce_tasks)
         pool = self.pool
         if pool is not None:
-            jcs.append('mapred.fairscheduler.pool=%s' % pool)
+            # Supporting two schedulers: fair (default) and capacity using the same option
+            scheduler_type = configuration.get_config().get('hadoop', 'scheduler', 'fair')
+            if scheduler_type == 'fair':
+                jcs.append('mapred.fairscheduler.pool=%s' % pool)
+            elif scheduler_type == 'capacity':
+                jcs.append('mapred.job.queue.name=%s' % pool)
         return jcs
 
 
